/* int01.cc       last Update: 25.8.2012
interrupt example for INT0 and INT1
 Version: 0.0
 Prozessor: Atmega8
 Circuit: Port B0,B1,B2 connected to LEDs (over 470 Ohm resistors)
          Port D2(INT0), D3(INT1) connected to buttons over 1k Ohm (or less)
          (resistors at inputs will protect the controller against bad programming mistakes)
 Autor: Rolf Pfister
 Copyright: Freeware
*/

#include <avr/io.h>
#include <avr/interrupt.h>

void interrupts_init()
{
 /* Table from datasheet chapter external interrupts:
    0  0   low level generates an interrupt
    0  1   any logical change generates an interrupt
    1  0   falling edge generates an interrupt
    1  1   raising edge generates an interrupt
 */
 MCUCR = (0<<ISC01) | (1<<ISC00)  //any logical change on INT0 generates an interrupt
       | (0<<ISC11) | (1<<ISC10); //any logical change on INT1 generates an interrupt
 //trying out other edges/levels:
 //MCUCR = (0<<ISC01) | (0<<ISC00)
 //      | (0<<ISC11) | (0<<ISC10);
 GICR = (1<<INT1) | (1<<INT0); //turn on interrupts for INT0 and INT1
}

// Variables used by interrupts and by main program must be defined with "volatile".
// This will turn off compiler optimisations for this variables.
volatile unsigned char button1=0,button2=0;

ISR(INT0_vect) //Interrupt SubRoutine for INT0
{
 button1=1;
}

ISR(INT1_vect) //Interrupt SubRoutine for INT1
{
 button2=1;
}

int main(void)
{
 unsigned int counter=0;
 DDRB  = 0x07; //PortB PB0-2 defining as output
 PORTB = 0; //all LEDs off
 DDRD  = 0x00; //PortD all inputs (not really necessairy because default anyway)
 PORTD = 0x0C; //set pullup resistors for PD2 and PD3
 interrupts_init();

 sei(); //Interrupts turning on

 while(1) //main loop
   {
    if(button1!=0)
     {
      PORTB |= 1;  //turn LED on
      PORTB &= ~2; //turn LED2 off
      button1 = 0;
     }
    if(button2!=0)
     {
      PORTB &= ~1; //turn LED off
      PORTB |= 2;  //turn LED2 on
      button2 = 0;
     }
    if(++counter==0) PORTB ^= 4; //LED3 blinking (will be slower when low level interrupts used)
   }

 return 0;
}
